package My::Schema::Result::User;
use strict;
use warnings;
use base 'DBIx::Class::Core';

__PACKAGE__->table('user');
__PACKAGE__->add_columns(
    id => {
        data_type         => 'INTEGER',
        is_nullable       => 0,
        is_auto_increment => 1,
    },

    username => {
        data_type   => 'VARCHAR',
        size        => 255,
        is_nullable => 0,
    },
);

__PACKAGE__->set_primary_key('id');
__PACKAGE__->add_unique_constraint([qw/username/]);

__PACKAGE__->has_one( profile => 'My::Schema::Result::UserProfile' );
__PACKAGE__->has_many( tweets => 'My::Schema::Result::Tweet', 'user' );

__PACKAGE__->has_many( following_maps => 'My::Schema::Result::FollowingMap', 'user' );
__PACKAGE__->has_many( follower_maps => 'My::Schema::Result::FollowingMap', 'target' );

__PACKAGE__->many_to_many( followings => following_maps => 'target' );
__PACKAGE__->many_to_many( followers  => follower_maps => 'user' );

sub insert {
    my $self = shift;

    # 新規ユーザー作成時に対応するUserProfileテーブルも作成する
    my $schema = $self->result_source->schema;
    my $txn    = $schema->txn_scope_guard;

    my $user = $self->next::method(@_);
    $user->create_related( profile => {} );

    $txn->commit;

    $user;
}

sub follow {
    my ($self, $target) = @_;

    die '$target is not User object'
        unless ref($target) and $target->isa('My::Schema::Result::User');

    my $already_follow = $self->following_maps({ target => $target->id })->first;
    unless ($already_follow) {
        $self->add_to_following_maps({ target => $target->id });
    };
}

1;

